@@ -15,6 +15,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
+#define PY_SSIZE_T_CLEAN
 #include <stdbool.h>
 #include <Python.h>
 #include <apr_general.h>
@@ -79,11 +80,6 @@ static void py_editor_dealloc(PyObject *self)
 	PyObject_Del(self);
 }
 
-/* paranoia check */
-#if defined(SIZEOF_SIZE_T) && SIZEOF_SIZE_T != SIZEOF_LONG
-#error "Unable to determine PyArg_Parse format for size_t"
-#endif
-
 /* svn_filesize_t is always 64 bits */
 #if SIZEOF_LONG == 8
 #define SVN_FILESIZE_T_PYFMT "k"
@@ -225,7 +221,7 @@ static PyObject *py_file_editor_change_prop(PyObject *self, PyObject *args)
 	EditorObject *editor = (EditorObject *)self;
 	char *name;
 	svn_string_t c_value;
-	int vallen;
+	Py_ssize_t vallen;
 
 	if (!PyArg_ParseTuple(args, "sz#", &name, &c_value.data, &vallen))
 		return NULL;
@@ -397,15 +393,15 @@ static PyObject *py_dir_editor_delete_entry(PyObject *self, PyObject *args)
 
 static PyObject *py_dir_editor_add_directory(PyObject *self, PyObject *args)
 {
-	PyObject *py_path;
+	PyObject *py_path, *py_copyfrom_path = Py_None;
 	const char *path;
-	char *copyfrom_path = NULL;
+	const char *copyfrom_path = NULL;
 	svn_revnum_t copyfrom_rev = -1;
 	void *child_baton;
 	EditorObject *editor = (EditorObject *)self;
 	apr_pool_t *subpool;
 
-	if (!PyArg_ParseTuple(args, "O|zl", &py_path, &copyfrom_path, &copyfrom_rev))
+	if (!PyArg_ParseTuple(args, "O|Ol", &py_path, &py_copyfrom_path, &copyfrom_rev))
 		return NULL;
 
 	if (editor->done) {
@@ -423,6 +419,13 @@ static PyObject *py_dir_editor_add_directory(PyObject *self, PyObject *args)
 		return NULL;
 	}
 
+	if (py_copyfrom_path != Py_None) {
+		copyfrom_path = py_object_to_svn_uri(py_copyfrom_path, editor->pool);
+		if (copyfrom_path == NULL) {
+			return NULL;
+		}
+	}
+
 	RUN_SVN(editor->editor->add_directory(
 		path, editor->baton,
 		copyfrom_path == NULL?NULL:svn_uri_canonicalize(copyfrom_path, editor->pool),
@@ -479,7 +482,7 @@ static PyObject *py_dir_editor_change_prop(PyObject *self, PyObject *args)
 	char *name;
 	svn_string_t c_value;
 	EditorObject *editor = (EditorObject *)self;
-	int vallen;
+	Py_ssize_t vallen;
 
 	if (!PyArg_ParseTuple(args, "sz#", &name, &c_value.data, &vallen))
 		return NULL;
@@ -563,14 +566,14 @@ static PyObject *py_dir_editor_absent_directory(PyObject *self, PyObject *args)
 static PyObject *py_dir_editor_add_file(PyObject *self, PyObject *args)
 {
 	const char *path;
-	char *copy_path=NULL;
-	PyObject *py_path;
+	const char *copy_path=NULL;
+	PyObject *py_path, *py_copy_path = Py_None;
 	svn_revnum_t copy_rev=-1;
 	void *file_baton = NULL;
 	EditorObject *editor = (EditorObject *)self;
 	apr_pool_t *subpool;
 
-	if (!PyArg_ParseTuple(args, "O|zl", &py_path, &copy_path, &copy_rev))
+	if (!PyArg_ParseTuple(args, "O|Ol", &py_path, &py_copy_path, &copy_rev))
 		return NULL;
 
 	if (editor->done) {
@@ -588,9 +591,15 @@ static PyObject *py_dir_editor_add_file(PyObject *self, PyObject *args)
 		return NULL;
 	}
 
+	if (py_copy_path != Py_None) {
+		copy_path = py_object_to_svn_uri(py_copy_path, editor->pool);
+		if (copy_path == NULL) {
+			return NULL;
+		}
+	}
+
 	RUN_SVN(editor->editor->add_file(path, editor->baton,
-		copy_path == NULL?NULL:svn_uri_canonicalize(copy_path, editor->pool),
-		copy_rev, editor->pool, &file_baton));
+		copy_path, copy_rev, editor->pool, &file_baton));
 
 	subpool = Pool(NULL);
 	if (subpool == NULL)
