@@ -17,31 +17,37 @@
 
 __author__ = "Jelmer Vernooij <jelmer@jelmer.uk>"
 
-from subvertpy import SubversionException, ERR_BAD_URL 
+from subvertpy import SubversionException, ERR_BAD_URL
 
 from subvertpy import _ra
-from subvertpy._ra import *
-from subvertpy import ra_svn
+from subvertpy._ra import *  # noqa: F403,F401
+from subvertpy import ra_svn  # noqa: F401
 
-import urllib
+try:
+    from urllib2 import splittype
+except ImportError:
+    from urllib.parse import splittype
 
 url_handlers = {
         "svn": _ra.RemoteAccess,
-#       "svn": ra_svn.Client,
+        # "svn": ra_svn.Client,
         "svn+ssh": _ra.RemoteAccess,
-#       "svn+ssh": ra_svn.Client,
+        # "svn+ssh": ra_svn.Client,
         "http": _ra.RemoteAccess,
         "https": _ra.RemoteAccess,
         "file": _ra.RemoteAccess,
 }
 
+
 def RemoteAccess(url, *args, **kwargs):
     """Connect to a remote Subversion server
 
     :param url: URL to connect to
     :return: RemoteAccess object
     """
-    (type, opaque) = urllib.splittype(url)
-    if not type in url_handlers:
+    if isinstance(url, bytes):
+        url = url.decode("utf-8")
+    (type, opaque) = splittype(url)
+    if type not in url_handlers:
         raise SubversionException("Unknown URL type '%s'" % type, ERR_BAD_URL)
     return url_handlers[type](url, *args, **kwargs)
